require "helper"
require "fluent/plugin/filter_dockergelf.rb"

class DockergelfFilterTest < Test::Unit::TestCase
  setup do
    Fluent::Test.setup
  end

  # default configuration for tests
  CONFIG = %[
    param1 value1
    param2 value2
  ]

  def create_driver(conf = CONFIG)
    Fluent::Test::Driver::Filter.new(Fluent::Plugin::DockergelfFilter).configure(conf)
  end

  def filterWithTime(config, time, message)
    d = create_driver(config)
    d.run() do
      d.feed("tag.input",time, message)
    end
    d.filtered_records
  end

  # sub_test_case 'configured with invalid configuration' do
  #   test 'empty configuration' do
  #     assert_raise(Fluent::ConfigError) do
  #       create_driver('')
  #     end
  #   end
  #
  #   test 'param1 should reject too short string' do
  #     conf = %[
  #       param1 a
  #     ]
  #     assert_raise(Fluent::ConfigError) do
  #       create_driver(conf)
  #     end
  #   end
  #   # ...
  # end

  sub_test_case 'plugin will handle gelf' do
    test 'parse gel log' do
      conf = CONFIG
      time = event_time('2018-10-20T00:00:00.123Z')
      logtime=Time.at(time.to_f).iso8601(3).to_s
      message = {"log" => '{"_instance_id":"abc","_microservice_id":"","host":"","level":4,"short_message":"somemsg","timestamp":1539143207.2754333,"version":"1.1","full_message":"aaa"}',
                  "stream" => 'stderr'
      }
      filtered_record = filterWithTime(conf, time, message)[0]
      puts filtered_record
      assert_equal(logtime, filtered_record["logtime"])
      assert_equal(4, filtered_record["level"])
      assert_equal("stderr", filtered_record["stream"])
      assert_equal("somemsg:aaa", filtered_record["log"])
      assert_equal("abc", filtered_record["instance_id"])
    end
  end

  sub_test_case 'plugin will handle nongelf' do
    test 'parse nonegelf log' do
      conf = CONFIG
      time = event_time('2018-10-20T00:00:00.123Z')
      logtime=Time.at(time.to_f).iso8601(3).to_s
      message = {"log" => 'abc',
                 "stream" => 'stderr'
      }
      filtered_record = filterWithTime(conf, time, message)[0]
      puts filtered_record
      assert_equal(logtime, filtered_record["logtime"])
      assert_equal("stderr", filtered_record["stream"])
      assert_equal("abc", filtered_record["log"])
    end
    test 'parse bad json log' do
      conf = CONFIG
      time = event_time('2018-10-20T00:00:00.123Z')
      logtime=Time.at(time.to_f).iso8601(3).to_s
      message = {"log" => '{"asdf":"aa",[}',
                 "stream" => 'stderr'
      }
      filtered_record = filterWithTime(conf, time, message)[0]
      puts filtered_record
      assert_equal(logtime, filtered_record["logtime"])
      assert_equal("stderr", filtered_record["stream"])
      assert_equal('{"asdf":"aa",[}', filtered_record["log"])
    end
  end
  #
  # sub_test_case 'plugin will add some fields' do
  #   test 'add hostname to record' do
  #     conf = CONFIG
  #     messages = [
  #         { "message" => "This is test message" }
  #     ]
  #     expected = [
  #         { "message" => "This is test message"}
  #     ]
  #     filtered_records = filter(conf, messages)
  #     assert_equal(expected, filtered_records)
  #   end

end
